@@ -888,7 +888,7 @@ myproxy_authenticate_accept_fqans(myproxy_socket_attrs_t *attrs, char *client_na
          GSI_SOCKET_ERROR)) {
         GSI_SOCKET_get_error_string(attrs->gsi_socket, error_string,
 	      			    sizeof(error_string));
-	verror_put_string("Error getting client VOMS attributes (%s). Continuing without attributes support.", error_string);
+        myproxy_debug("No client VOMS attributes (%s). Continuing without attributes support.\n", error_string);
     }
 
     return 0;
@@ -964,6 +964,18 @@ myproxy_accept_delegation_ex(myproxy_socket_attrs_t *attrs, char **credentials,
   return 0;
 }
 
+int
+myproxy_request_cert(myproxy_socket_attrs_t *attrs, char *certreq,
+                     char **credentials, int *credential_len)
+{
+  assert(attrs);
+  assert(certreq);
+  GSI_SOCKET_delegation_set_certreq(attrs->gsi_socket, certreq);
+
+  return myproxy_accept_delegation_ex(attrs, credentials,
+                                      credential_len, NULL);
+}
+
 int
 myproxy_serialize_request(const myproxy_request_t *request, char *data,
 			  const int datalen) 
